searchData={"items":[{"type":"module","title":"Niffler","doc":"Just-In-Time nif generator, FFI generator, C-compiler based on TinyCC.\nFor Linux, MacOS, Windows (msys2)\n\n# Using Niffler\n\nOnce installed, you can use the Niffler to define new nif modules using embedded C fragments:\n\n```\ndefmodule Example do\n  use Niffler\n\n  defnif :count_zeros, [str: :binary], [ret: :int] do\n    \"\"\"\n    while($str.size--) {\n      if (*$str.data++ == 0) $ret++;\n    }\n    \"\"\"\n  end\nend\n\n{:ok, [2]} = Example.count_zeros(<<0,1,0>>)\n```\n\nSee `Niffler.defnif/4` for a full explanation of the parameters passed.","ref":"Niffler.html"},{"type":"module","title":"Variable binding in C fragments - Niffler","doc":"Each c fragment is wrapped in shortcut macros to receive direct access to the defined parameters. So when  defining a nif:\n\n```\n  defnif :count_zeros, [str: :binary], [ret: :int] do\n  #       ^name         ^input          ^output\n```\n\nThere will be macros defined for each defined input and output variable. Each macro is prefixed with a dollar sign `$` to highlight the fact that it's a macro and not real variable:\n\nIn the exampe above:\n\n* `$str` will be defined as a macro alias pointing to a binary type\n* `$ret` will be defined as a macro alias to an integer\n\nInput and output variables are allocated on the stack of the nif function call making them thread-safe and isolated.","ref":"Niffler.html#module-variable-binding-in-c-fragments"},{"type":"module","title":"Working with binaries - Niffler","doc":"When using binaries as input or output they are returned as as a struct with two members:\n\n```c++\n  typedef struct {\n    uint64_t size;\n    unsigned char* data;\n  } Binary;\n```\n\nThe size and data fields can be used to read from inputs and write to outputs.\n\n*Warning:* __NEVER__ write to input binaries. These are pointers into the BEAM VM, changing their values will have unknown but likely horrible consequences.\n\nConstructing output binaries requires care. The easiest way is to use the built-in macro function `$alloc(size_t)` which allows to allocate memory temporary during the runtime of the nif, that will be automatically freed. Other possibilities are to use the system `malloc()` directly but then `free()` needs to be called at a later point in time, or to use static memory in the module. Stack variables (or from `alloca`) don't work as they are being destroyed before the Niffler program returns and the result values are beeing read. Two examples that are possible here:\n\n```\ndefmodule BinaryExample\n  use Niffler\n\n  defnif :static, [], [ret: :binary] do\n    \"\"\"\n    static char str[10];\n    $ret.size = sizeof(str);\n    $ret.data = str;\n    for (int i = 0; i  >]} = BinaryExample.static()\n\n{:ok, [<<3, 2, 1>>]} = BinaryExample.reverse(<<1, 2, 3>>)\n```","ref":"Niffler.html#module-working-with-binaries"},{"type":"module","title":"Concurrency - Niffler","doc":"Each generated Niffler function is a small c program in its own call stack. Multiple\nruns of the same program are all executed in the same context. This allows to keep state in\nc programs when needed but also increases the chances for concurrency issues.\n\nLets take this stateful counter:\n\n```\n  defnif :counter, [], [ret: :int] do\n    \"\"\"\n    static uint64_t counter = 0;\n    $ret = counter++;\n    \"\"\"\n  end\n```\n\nThe niffler protects against certain types concurrency issues by creating all input and output variables on the call stack. Two functions calls at the same time do never interefere on the input and output parameters.\n\nBut this protection is not true for static and global variables. The above counter example is using a static counter variable, this variable will be the same instance on each call. So with high concurrency these things could happen:\n\n* counter returns the same value for two concurrent calls.\n* counter skips a value for two concurrent calls.\n\nThe same problem affects the static binary example above. When called multiple times concurrently it will overwrite the static variable multiple times return undefined results.","ref":"Niffler.html#module-concurrency"},{"type":"module","title":"Defining helper functions - Niffler","doc":"When using `Niffler.defnif/4` you sometimes might want to create helper functions\nor defines outside the function body. For short fragements it's possible to use the\n`DO_RUN` and `END_RUN` macros to separate the function body from global helpers:\n\nHere an example defining a recursive fibonacci function. In order to refer to the\nfunction name `fib()` recursively in c it needs to be defined. So we define it globally\noutside of an explicity `DO_RUN` / `END_RUN` block:\n\n```\n  defnif :fib, [a: :int], ret: :int do\n  \"\"\"\n  int64_t fib(int64_t f) {\n    if (f < 2) return 1;\n    return fib(f-1) + fib(f-2);\n  }\n\n  DO_RUN\n    $ret = fib($a);\n  END_RUN\n  \"\"\"\nend\n```\n\nInterally `DO_RUN` and `END_RUN` are c-macros that will be converted to the correct\nniffler wrapping to execute the code, while anything outside the `DO_RUN` / `END_RUN`\nblock will be copied into the c code without modification.\n\nFor larger blocks it might though be better to use `Niffler.Library` and override the\n`Niffler.Library.c:header/0` callback.","ref":"Niffler.html#module-defining-helper-functions"},{"type":"module","title":"Using shared libraries (.dll, .so, .dylib) - Niffler","doc":"For working with shared library and create foreign function interfaces (FFI) for those\nplease look at `Niffler.Library`","ref":"Niffler.html#module-using-shared-libraries-dll-so-dylib"},{"type":"module","title":"Standard library - Niffler","doc":"Niffler comes with a minimal c standard library. Please check standard c\ndocumentation for reference. This is just a list of defined functions and types:\n\n```\n  /* types */\n  typedef signed char         int8_t;\n  typedef unsigned char       uint8_t;\n  typedef short               int16_t;\n  typedef unsigned short      uint16_t;\n  typedef int                 int32_t;\n  typedef unsigned            uint32_t;\n  typedef long long           int64_t;\n  typedef unsigned long long  uint64_t;\n\n  typedef struct {\n    uint64_t size;\n    unsigned char* data;\n  } Binary;\n\n  /* niffler helper */\n  void *$alloc(size_t size);\n\n  /* stdarg.h */\n  typedef __builtin_va_list va_list;\n\n  /* stddef.h */\n  typedef __SIZE_TYPE__ size_t;\n  typedef __PTRDIFF_TYPE__ ssize_t;\n  typedef __WCHAR_TYPE__ wchar_t;\n  typedef __PTRDIFF_TYPE__ ptrdiff_t;\n  typedef __PTRDIFF_TYPE__ intptr_t;\n  typedef __SIZE_TYPE__ uintptr_t;\n  void *alloca(size_t size);\n\n  /* stdlib.h */\n  void *calloc(size_t nmemb, size_t size);\n  void *malloc(size_t size);\n  void free(void *ptr);\n  void *realloc(void *ptr, size_t size);\n  int atoi(const char *nptr);\n  long int strtol(const char *nptr, char **endptr, int base);\n  unsigned long int strtoul(const char *nptr, char **endptr, int base);\n  void exit(int);\n\n  /* stdio.h */\n  extern FILE *stdin;\n  extern FILE *stdout;\n  extern FILE *stderr;\n  FILE *fopen(const char *path, const char *mode);\n  FILE *fdopen(int fildes, const char *mode);\n  FILE *freopen(const  char *path, const char *mode, FILE *stream);\n  int fclose(FILE *stream);\n  size_t  fread(void *ptr, size_t size, size_t nmemb, FILE *stream);\n  size_t  fwrite(void *ptr, size_t size, size_t nmemb, FILE *stream);\n  int fgetc(FILE *stream);\n  int fputs(const char *s, FILE *stream);\n  char *fgets(char *s, int size, FILE *stream);\n  int getc(FILE *stream);\n  int getchar(void);\n  char *gets(char *s);\n  int ungetc(int c, FILE *stream);\n  int fflush(FILE *stream);\n  int putchar(int c);\n\n  int printf(const char *format, ...);\n  int fprintf(FILE *stream, const char *format, ...);\n  int sprintf(char *str, const char *format, ...);\n  int snprintf(char *str, size_t size, const  char  *format, ...);\n  int asprintf(char **strp, const char *format, ...);\n  int vprintf(const char *format, va_list ap);\n  int vfprintf(FILE  *stream,  const  char *format, va_list ap);\n  int vsprintf(char *str, const char *format, va_list ap);\n  int vsnprintf(char *str, size_t size, const char  *format, va_list ap);\n  int vasprintf(char  **strp,  const  char *format, va_list ap);\n\n  void perror(const char *s);\n\n  /* string.h */\n  char *strcat(char *dest, const char *src);\n  char *strchr(const char *s, int c);\n  char *strrchr(const char *s, int c);\n  char *strcpy(char *dest, const char *src);\n  void *memcpy(void *dest, const void *src, size_t n);\n  void *memmove(void *dest, const void *src, size_t n);\n  void *memset(void *s, int c, size_t n);\n  char *strdup(const char *s);\n  size_t strlen(const char *s);\n\n  /* dlfcn.h */\n  void *dlopen(const char *filename, int flag);\n  const char *dlerror(void);\n  void *dlsym(void *handle, char *symbol);\n  int dlclose(void *handle);\n```","ref":"Niffler.html#module-standard-library"},{"type":"function","title":"Niffler.compile/3","doc":"Low level function takes a string as input and compiles it into a nif program. Returning the program\nreference. Prefer using the high-level function `Niffler.defnif/4` or `Niffler.Library` instead.","ref":"Niffler.html#compile/3"},{"type":"function","title":"Examples - Niffler.compile/3","doc":"iex> {:ok, prog} = Niffler.compile(\"$ret = $a * $b;\", [a: :int, b: :int], [ret: :int])\n    iex> Niffler.run(prog, [3, 4])\n    {:ok, [12]}\n\n    iex> code = \"for (int i = 0; i < $str.size; i++) if ($str.data[i] == 0) $ret++;\"\n    iex> {:ok, prog} = Niffler.compile(code, [str: :binary], [ret: :int])\n    iex> Niffler.run(prog, [<<0,1,1,0,1,5,0>>])\n    {:ok, [3]}","ref":"Niffler.html#compile/3-examples"},{"type":"macro","title":"Niffler.defnif/4","doc":"Defines a new nif member method. To use defnif() import Niffler into\nyour module with `use Niffler`.\n\ndefnif takes three parameters and a c-fragment function body:\n\n* `name` - an atom, the name of the to be defined nif function\n* `inputs` - a keyword list of the format `[name: type]`\n* `outputs` - a keyword list of the format `[name: type]`\n\nThe `inputs` and `outputs` keyword lists take atom() as names and types.\nThe parameter names can be freely choosen* the currently supported\ntypes are:\n\n* `int` or `int64` - a signed 64-bit integer\n* `uint64` - an unsigned 64-bit integer\n* `double` - a double (floating point number)\n* `binary` - an Elixir binary/string\n\n```\ndefmodule Example do\n  use Niffler\n\n  defnif :count_zeros, [str: :binary], [ret: :int] do\n    \"\"\"\n    while($str.size--) {\n      if (*$str.data++ == 0) $ret++;\n    }\n    \"\"\"\n  end\nend\n\n{:ok, [2]} = Example.count_zeros(<<0,1,0>>)\n```","ref":"Niffler.html#defnif/4"},{"type":"function","title":"Niffler.run/3","doc":"Low level function that executes the given Niffler program and\nreturns any output values. Prefer using the high-level function\n`Niffler.defnif/4` or `Niffler.Library` instead.","ref":"Niffler.html#run/3"},{"type":"function","title":"Examples - Niffler.run/3","doc":"iex> {:ok, prog} = Niffler.compile(\"$ret = $a << 2;\", [a: :int], [ret: :int])\n    iex> Niffler.run(prog, [5])\n    {:ok, [20]}","ref":"Niffler.html#run/3-examples"},{"type":"behaviour","title":"Niffler.Library","doc":"`Niffler.Library` allows to wrap dynamically loaded libraries. And create foreign\nfunction interfaces (FFI) for them. This usually requires:\n\n* Shared headers such for type definitions\n* Some init code involving `dlopen()`\n* Static variables/state that is part of the library\n* Potential deinit code involving `dlclose()`\n\n`Niffler.Library` allows to produce modules at runtime doing all these\nthings and exposing access to the c-functions of a dynamic library.\n\n```\ndefmodule Gmp do\n  use Niffler.Library, thread_safe: false\n\n  @impl true\n  def header() do\n    \"\"\"\n    // library handle\n    void *gmp;\n\n    // types\n    typedef struct\n    {\n      int _mp_alloc;\n      int _mp_size;\n      void *_mp_d;\n    } __mpz_struct;\n\n    typedef __mpz_struct mpz_t[1];\n    void (*mpz_init)(mpz_t);\n    // ...\n    \"\"\"\n  end\n\n  @impl true\n  def on_load() do\n    \"\"\"\n    // loading the library with dlopen()\n    gmp = dlopen(\"libgmp.#{library_suffix()}\", RTLD_LAZY);\n    if (!gmp) {\n      return \"could not load libgmp\";\n    }\n    // loading symbols:\n    dlerror();\n    if (!(mpz_init = dlsym(gmp, \"__gmpz_init\"))) {\n      return dlerror();\n    }\n    // other symbols ...\n    \"\"\"\n  end\n\n  @impl true\n  def on_destroy() do\n    \"\"\"\n    if (!gmp) {\n      return;\n    }\n    // unloading\n    dlclose(gmp);\n    \"\"\"\n  end\n\n  # definining one or more operations here...\n  defnif :mul, [a: :int, b: :int], ret: :int do\n    \"\"\"\n    mpz_set_si(ma, $a);\n    mpz_set_si(mb, $b);\n    mpz_mul(mc, ma, mb);\n    $ret = mpz_get_si(mc);\n    \"\"\"\n  end\nend\n```\n\nOnce defined the module functions can be used via:\n\n```\n  {ok, [result]} = Gmp.mul(4, 5)\n```","ref":"Niffler.Library.html"},{"type":"function","title":"Niffler.Library.at_exit/1","doc":"","ref":"Niffler.Library.html#at_exit/1"},{"type":"macro","title":"Niffler.Library.attach_function/3","doc":"","ref":"Niffler.Library.html#attach_function/3"},{"type":"macro","title":"Niffler.Library.callback/3","doc":"","ref":"Niffler.Library.html#callback/3"},{"type":"macro","title":"Niffler.Library.defnif/4","doc":"Defines a new nif function in the current module.\n\n  Same as `Niffler.defnif/4` but with access to the current module context.","ref":"Niffler.Library.html#defnif/4"},{"type":"macro","title":"Niffler.Library.ffi_lib/1","doc":"","ref":"Niffler.Library.html#ffi_lib/1"},{"type":"callback","title":"Niffler.Library.header/0","doc":"Return a c-fragement of a common header for type definitions,\n  static variables and other neccesary c code.\n\n  Example:\n\n  ```\n  @impl true\n  def header() do\n    \"\"\"\n    // library handle\n    void *gmp;\n    \"\"\"\n  end\n  ```","ref":"Niffler.Library.html#c:header/0"},{"type":"function","title":"Niffler.Library.library_suffix/0","doc":"Returns the current platforms default library suffix:\n\n  * `dll` on windows\n  * `dylib` on mac\n  * `so` on linux\n\n  Useful for dlopen() code to load the correct library:\n\n  ```\n  @impl true\n  def on_load() do\n    \"\"\"\n    // loading the library with dlopen()\n    gmp = dlopen(\"libgmp.#{library_suffix()}\", RTLD_LAZY);\n    if (!gmp) {\n      return \"could not load libgmp\";\n    }\n    \"\"\"\n  end\n  ```","ref":"Niffler.Library.html#library_suffix/0"},{"type":"callback","title":"Niffler.Library.on_destroy/0","doc":"Return a c-fragement that is called when the module is unloaded.\n  Typicallly this fragment would contain a call to `dlclose()` for\n  a dynamic library.\n\n  Example:\n\n  ```\n  @impl true\n  def on_destroy() do\n    \"\"\"\n    if (!gmp) {\n      return;\n    }\n    // unloading\n    dlclose(gmp);\n    \"\"\"\n  end\n  ```","ref":"Niffler.Library.html#c:on_destroy/0"},{"type":"callback","title":"Niffler.Library.on_load/0","doc":"Return a c-fragement that is called on the first call to the module.\n  Typicall this fragment would contain a call to `dlopen()` when loading\n  a dynamic library.\n\n  This c-fragment should return a char* (a common string in c) when any\n  error has occured.\n\n  Example:\n\n  ```\n  @impl true\n  def on_load() do\n    \"\"\"\n    // loading the library with dlopen()\n    gmp = dlopen(\"libgmp.#{library_suffix()}\", RTLD_LAZY);\n    if (!gmp) {\n      return \"could not load libgmp\";\n    }\n    \"\"\"\n  end\n  ```","ref":"Niffler.Library.html#c:on_load/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}